//========================================================================================
//
//	File:				FWColor.h
//	Release Version:	$ ODF 1 $
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWCOLOR_H
#define FWCOLOR_H

#ifndef SLCOLOR_H
#include "SLColor.h"
#endif

// ----- Foundation Includes -----

#ifndef FWSTDDEF_H
#include <FWStdDef.h>
#endif

// ----- Macintosh Includes -----

#if defined(FW_BUILD_MAC) && !defined(__QUICKDRAW__)
#include <QuickDraw.h>
#endif

// ----- Windows Includes -----

#if defined(FW_BUILD_WIN) && !defined(_INC_WINDOWS)
#include <windows.h>
#endif

//========================================================================================
//	Forward Class Declarations
//========================================================================================

class FW_CColor;
class FW_CWritableStream;
class FW_CReadableStream;

//========================================================================================
//	Globale operators << and >>
//========================================================================================

FW_CWritableStream& operator<<(FW_CWritableStream& stream, const FW_CColor& color);
FW_CReadableStream& operator>>(FW_CReadableStream& stream, FW_CColor& color);

//========================================================================================
//	class FW_CColor
//========================================================================================

class FW_CColor : public FW_SColor
{
//----------------------------------------------------------------------------------------
//	Constructors/Destructors
//
public:
	FW_CColor();
	FW_CColor(const FW_CColor& color);
	FW_CColor(unsigned short r, unsigned short g, unsigned short b);
	FW_CColor(FW_CReadableStream& stream);

	FW_CColor(FW_SColor rgb);

#ifdef FW_BUILD_MAC
	FW_CColor(const RGBColor* rgb);
	FW_CColor(const RGBColor& rgb);
#endif

#ifdef FW_BUILD_WIN
	FW_CColor(COLORREF colorref);
#endif

//----------------------------------------------------------------------------------------
//	Operators
//
public:
#ifdef FW_BUILD_MAC	
	FW_Boolean operator==(const RGBColor& rgb) const;
	FW_Boolean operator!=(const RGBColor& rgb) const;
#endif

	// ----- Assignment operator -----	
	FW_CColor& operator=(const FW_CColor& color);
	FW_CColor& operator=(FW_SColor color);
	
#ifdef FW_BUILD_MAC
	FW_CColor& operator=(const RGBColor& rgb);
	operator RGBColor() const;
#endif
	
#ifdef FW_BUILD_WIN
	FW_CColor& operator=(RGBQUAD quad);
	operator RGBQUAD() const;

	FW_CColor& operator=(RGBTRIPLE triple);
	operator RGBTRIPLE() const;
	
	FW_CColor& operator=(COLORREF colorref);
	operator COLORREF() const;
#endif

	// ----- Arithmetic, by another color -----
	FW_CColor operator+(const FW_CColor& color) const;
	FW_CColor operator-(const FW_CColor& color) const;

	FW_CColor& operator+=(const FW_CColor& color);
	FW_CColor& operator-=(const FW_CColor& color);

	// ----- Arithmetic by a scalar -----

	FW_CColor operator+(unsigned short a) const;
	FW_CColor operator-(unsigned short a) const;
	FW_CColor operator*(unsigned short a) const;
	FW_CColor operator/(unsigned short a) const;

	FW_CColor& operator+=(unsigned short a);
	FW_CColor& operator-=(unsigned short a);
	FW_CColor& operator*=(unsigned short a);
	FW_CColor& operator/=(unsigned short a);

//----------------------------------------------------------------------------------------
//	New public API
//
public:
	// ----- Set/Get RGB Component -----
	void		SetRGB(unsigned short r, unsigned short g, unsigned short b);
	void		GetRGB(unsigned short& r, unsigned short& g, unsigned short& b) const;
	
	FW_Boolean 	IsDarkerThan(const FW_CColor& color) const;
	FW_Boolean 	IsLighterThan(const FW_CColor& color) const;

	// ----- Persistence -----
	void		Read(FW_CReadableStream& stream);
	void		Write(FW_CWritableStream& stream) const;
	
//----------------------------------------------------------------------------------------
//	Private API
//
private:
	unsigned long 	Brightness() const;

	static unsigned short	AddComponents(unsigned short a, unsigned short b);
	static unsigned short	SubtractComponents(unsigned short a, unsigned short b);
	static unsigned short	MultiplyComponents(unsigned short a, unsigned short b);
	static unsigned short	DivideComponents(unsigned short a, unsigned short b);
};

//========================================================================================
//	inlines
//========================================================================================

#ifdef FW_BUILD_MAC
//----------------------------------------------------------------------------------------
//	FW_CColor::operator!=
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CColor::operator!=(const RGBColor& rgb) const
{
	return !(*this == rgb);
}
#endif

#ifdef FW_BUILD_WIN
//----------------------------------------------------------------------------------------
//	FW_CColor::operator=
//----------------------------------------------------------------------------------------

inline FW_CColor& FW_CColor::operator=(COLORREF color)
{
	fRGB = color;
	return *this;
}
#endif

#ifdef FW_BUILD_WIN
//----------------------------------------------------------------------------------------
//	FW_CColor::operator COLORREF
//----------------------------------------------------------------------------------------

inline FW_CColor::operator COLORREF() const
{
	return fRGB;
}
#endif

#endif
