//========================================================================================
//
//	File:				SLShAttr.h
//	Release Version:	$ ODF 1 $
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef SLSHATTR_H
#define SLSHATTR_H

#ifndef FWSTDDEF_H
#include "FWStdDef.h"
#endif

#ifndef SLSTRREP_H
#include "SLStrRep.h"
#endif

#ifndef FWFXMATH_H
#include "FWFxMath.h"
#endif

#ifndef SLGCONST_K
#include "SLGConst.k"
#endif

#ifndef SLGRDEF_H
#include "SLGrDef.h"
#endif

#ifndef SLCOLOR_H
#include "SLColor.h"
#endif

#ifndef SLPAT_H
#include "SLPat.h"
#endif

#ifndef SLSTRMRW_H
#include "SLStrmRW.h"
#endif

//========================================================================================
// Pattern
//========================================================================================

class FW_CPrivPatternRep;
class FW_CPrivFontRep;
class FW_CPrivInkRep;
class FW_CPrivStyleRep;

typedef FW_CPrivPatternRep* FW_HPattern;
typedef FW_CPrivInkRep* FW_HInk;
typedef FW_CPrivStyleRep* FW_HStyle;
typedef FW_CPrivFontRep* FW_HFont;

// Export or Import functions for CFM-68K [sfu]

#if defined(FW_ODFLIB_IMPORT)
#pragma import on
#elif defined(FW_ODFLIB)
#pragma export on
#endif

FW_EXTERN_C_BEGIN

// Creation

FW_HPattern			SL_API	FW_PrivPattern_CreateBlack(FW_PlatformError* error);
FW_HPattern			SL_API	FW_PrivPattern_CreateBW(const FW_BitPattern& bits, FW_PlatformError* error);
FW_HPattern			SL_API	FW_PrivPattern_CreateColor(const FW_PixelPattern& pixels, short nbColors, const FW_SColor* colorTable, FW_PlatformError* error);
FW_HPattern			SL_API	FW_PrivPattern_CreateFromPlatformColorPattern(FW_PlatformColorPattern pattern, FW_PlatformError* error);
FW_HPattern			SL_API	FW_PrivPattern_Copy(FW_HPattern pattern, FW_PlatformError* error);

// Reference counting

void				SL_API	FW_PrivPattern_Acquire(FW_HPattern pattern);
long				SL_API	FW_PrivPattern_GetRefCount(FW_HPattern pattern);
void				SL_API	FW_PrivPattern_Release(FW_HPattern pattern);

// Comparison

FW_Boolean			SL_API	FW_PrivPattern_IsEqual(FW_HPattern pattern, FW_HPattern pattern2);

// Operations

FW_PlatformError	SL_API	FW_PrivPattern_Invert(FW_HPattern pattern);
FW_PlatformError	SL_API	FW_PrivPattern_FlipHorizontally(FW_HPattern pattern);
FW_PlatformError	SL_API	FW_PrivPattern_FlipVertically(FW_HPattern pattern);
FW_PlatformError	SL_API	FW_PrivPattern_ShiftUp(FW_HPattern pattern);
FW_PlatformError	SL_API	FW_PrivPattern_ShiftDown(FW_HPattern pattern);
FW_PlatformError	SL_API	FW_PrivPattern_ShiftLeft(FW_HPattern pattern);
FW_PlatformError	SL_API	FW_PrivPattern_ShiftRight(FW_HPattern pattern);

// Streaming

FW_HPattern			SL_API	FW_PrivPattern_Read(FW_HReadableStream stream, FW_PlatformError* error);
void				SL_API	FW_PrivPattern_Write(FW_HPattern pattern, FW_HWritableStream stream, FW_PlatformError* error);

//========================================================================================
// Ink
//========================================================================================

// Creation

FW_HInk				SL_API	FW_PrivInk_Create(FW_SColor fore, FW_SColor back, FW_TransferModes mode, FW_PlatformError* error);
FW_HInk				SL_API	FW_PrivInk_CreateStandard(FW_EStandardInks std, FW_PlatformError* error);
FW_HInk				SL_API	FW_PrivInk_Copy(FW_HInk ink, FW_PlatformError* error);

// Reference counting

void				SL_API	FW_PrivInk_Acquire(FW_HInk ink);
long				SL_API	FW_PrivInk_GetRefCount(FW_HInk ink);
void				SL_API	FW_PrivInk_Release(FW_HInk ink);

// Attributes

FW_SColor			SL_API	FW_PrivInk_GetForeColor(FW_HInk ink);
FW_SColor			SL_API	FW_PrivInk_GetBackColor(FW_HInk ink);
FW_TransferModes	SL_API	FW_PrivInk_GetTransferMode(FW_HInk ink);

// Comparison

FW_Boolean			SL_API	FW_PrivInk_IsEqual(FW_HInk ink, FW_HInk ink2);

// Operations

void				SL_API	FW_PrivInk_SetForeColor(FW_HInk ink, FW_SColor color);
void				SL_API	FW_PrivInk_SetBackColor(FW_HInk ink, FW_SColor color);
void				SL_API	FW_PrivInk_SetTransferMode(FW_HInk ink, FW_TransferModes mode);

// Streaming

FW_HInk				SL_API	FW_PrivInk_Read(FW_HReadableStream hStream, FW_PlatformError* error);
void				SL_API	FW_PrivInk_Write(FW_HInk ink, FW_HWritableStream stream, FW_PlatformError* error);

//========================================================================================
// Style
//========================================================================================

// Creation

FW_HStyle			SL_API	FW_PrivStyle_CreateDash(FW_Fixed penSize, FW_EStyleDash dash, FW_PlatformError* error);
FW_HStyle			SL_API	FW_PrivStyle_CreatePattern(FW_Fixed penSize, FW_HPattern pattern, FW_PlatformError* error);
FW_HStyle			SL_API	FW_PrivStyle_CreateStandard(FW_EStandardStyles std, FW_PlatformError* error);
FW_HStyle			SL_API	FW_PrivStyle_Copy(FW_HStyle style, FW_PlatformError* error);

// Reference counting

void				SL_API	FW_PrivStyle_Acquire(FW_HStyle style);
long				SL_API	FW_PrivStyle_GetRefCount(FW_HStyle style);
void				SL_API	FW_PrivStyle_Release(FW_HStyle style);

// Attributes

FW_Fixed			SL_API	FW_PrivStyle_GetPenSize(FW_HStyle style);
FW_HPattern			SL_API	FW_PrivStyle_GetPattern(FW_HStyle style);
FW_HPattern			SL_API	FW_PrivStyle_GetUnSharedPattern(FW_HStyle style);
FW_EStyleDash		SL_API	FW_PrivStyle_GetDashStyle(FW_HStyle style);

// Comparison

FW_Boolean			SL_API	FW_PrivStyle_IsEqual(FW_HStyle style, FW_HStyle style2);

// Operations

void				SL_API	FW_PrivStyle_SetPenSize(FW_HStyle style, FW_Fixed penSize);
FW_PlatformError	SL_API	FW_PrivStyle_SetPattern(FW_HStyle style, FW_HPattern pattern);
void				SL_API	FW_PrivStyle_SetDashStyle(FW_HStyle style, FW_EStyleDash dash);

// Streaming

FW_HStyle			SL_API	FW_PrivStyle_Read(FW_HReadableStream hStream, FW_PlatformError* error);
void				SL_API	FW_PrivStyle_Write(FW_HStyle style, FW_HWritableStream stream, FW_PlatformError* error);

//========================================================================================
// Font
//========================================================================================

// Creation

FW_HFont			SL_API	FW_PrivFont_Create(FW_HString fontName, FW_FontStyle fontStyle, FW_Fixed fontSize, FW_PlatformError* error);
FW_HFont			SL_API	FW_PrivFont_CreateStandard(FW_EStandardFonts std, FW_PlatformError* error);
FW_HFont			SL_API	FW_PrivFont_Copy(FW_HFont font, FW_PlatformError* error);

// Reference counting

void				SL_API	FW_PrivFont_Acquire(FW_HFont font);
long				SL_API	FW_PrivFont_GetRefCount(FW_HFont font);
void				SL_API	FW_PrivFont_Release(FW_HFont font);

// Attributes

FW_Fixed			SL_API	FW_PrivFont_GetSize(FW_HFont font);
FW_FontStyle		SL_API	FW_PrivFont_GetStyle(FW_HFont font);
void				SL_API	FW_PrivFont_GetName(FW_HFont font, FW_HString* fontName, FW_PlatformError* error);

// Comparison

FW_Boolean			SL_API	FW_PrivFont_IsEqual(FW_HFont font, FW_HFont font2);

// Operations

void				SL_API	FW_PrivFont_SetSize(FW_HFont font, FW_Fixed size);
void				SL_API	FW_PrivFont_SetStyle(FW_HFont font, FW_FontStyle style);
void				SL_API	FW_PrivFont_SetName(FW_HFont font, FW_HString fontName, FW_PlatformError* error);

// Streaming

FW_HFont			SL_API	FW_PrivFont_Read(FW_HReadableStream hStream, FW_PlatformError* error);
void				SL_API	FW_PrivFont_Write(FW_HFont font, FW_HWritableStream stream, FW_PlatformError* error);

// Macintosh-only

#ifdef FW_BUILD_MAC
short				SL_API	FW_PrivFont_MacGetFontID(FW_HFont font);
Style				SL_API	FW_PrivFont_MacGetFontStyle(FW_HFont font);
#endif

FW_EXTERN_C_END

// For CFM-68K [sfu]

#if defined(FW_ODFLIB_IMPORT)
#pragma import off
#elif defined(FW_ODFLIB)
#pragma export off
#endif

#endif // SLSHATTR_H
