//========================================================================================
//
//	File:				SLPictur.cpp
//	Release Version:	$ ODF 1 $
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWOS.hpp"

#ifndef SLPICTUR_H
#include "SLPictur.h"
#endif

#ifndef PRPICTUR_H
#include "PRPictur.h"
#endif

#ifndef FWODEXCE_H
#include "FWODExce.h"
#endif

#ifndef FWSTRMRW_H
#include "FWStrmRW.h"
#endif

//========================================================================================
// Picture
//========================================================================================

#ifdef FW_BUILD_MAC
#pragma segment FWGraphics_Picture
#endif

// Creation

FW_HPicture			SL_API	FW_PrivPicture_CreateFromResource(
								FW_OResourceFile* 	resourceFile,
								FW_ResourceId 		resId,
								FW_PlatformError* 	error)
{
	FW_ERR_TRY
	{
		return FW_NEW(FW_CPrivPictureRep, (resourceFile, resId));
	}
	FW_ERR_CATCH
	return 0;	
}

FW_HPicture			SL_API	FW_PrivPicture_CreateFromPlatformPict(
								FW_PlatformPict 	platformPict,
								FW_PlatformError* 	error)
{
	FW_ERR_TRY
	{
		return FW_NEW(FW_CPrivPictureRep, (platformPict));
	}
	FW_ERR_CATCH
	return 0;	
}

FW_HPicture			SL_API	FW_PrivPicture_Copy(FW_HPicture picture, FW_PlatformError* 	error)
{
	FW_ERR_TRY
	{
		return FW_NEW(FW_CPrivPictureRep, (*picture));
	}
	FW_ERR_CATCH
	return 0;	
}

// Reference counting

void				SL_API	FW_PrivPicture_Acquire(FW_HPicture picture)
{
	// No try block necessary - Do not throw
	picture->Acquire();
}

long				SL_API	FW_PrivPicture_GetRefCount(FW_HPicture picture)
{
	// No try block necessary - Do not throw
	return picture->GetRefCount();
}

void				SL_API	FW_PrivPicture_Release(FW_HPicture picture)
{
	// No try block necessary - Do not throw
	picture->Release();
}

FW_Boolean			SL_API	FW_PrivPicture_IsEqual(FW_HPicture picture, FW_HPicture picture2)
{
	// No try block necessary - Do not throw
	return picture->IsEqual(*picture2);
}

// Bounds

void				SL_API	FW_PrivPicture_GetPictBounds(
								FW_HPicture 		picture,
								FW_SRect& 			bounds)
{
	// No try block necessary - Do not throw
	picture->GetPictBounds(bounds);
}

void				SL_API	FW_PrivPicture_GetPictBoundsGC(Environment* ev,
								FW_HPicture 		picture,
								FW_SGraphicContext&	gc,
								FW_SRect&			bounds)
{
	// No try block necessary - Do not throw
	picture->GetPictBoundsGC(ev, gc, bounds);
}

FW_PlatformPict		SL_API	FW_PrivPicture_GetPlatformPict(FW_HPicture picture)
{
	// No try block necessary - Do not throw
	return picture->GetPlatformPict();
}

FW_Boolean			SL_API	FW_PrivPicture_IsPlatformPictOrphan(FW_HPicture picture)
{
	// No try block necessary - Do not throw
	return picture->IsPlatformPictOrphan();
}

FW_PlatformPict		SL_API	FW_PrivPicture_OrphanPlatformPict(FW_HPicture picture)
{
	// No try block necessary - Do not throw
	return picture->OrphanPlatformPict();
}
	
FW_PlatformError	SL_API	FW_PrivPicture_SetPlatformPict(FW_HPicture picture, FW_PlatformPict newPict)
{
	// No try block necessary - Do not throw
	picture->SetPlatformPict(newPict);
	return FW_xNoError;
}

FW_PlatformError	SL_API	FW_PrivPicture_AdoptPlatformPict(FW_HPicture picture, FW_PlatformPict newPict)
{
	// No try block necessary - Do not throw
	picture->AdoptPlatformPict(newPict);
	return FW_xNoError;
}

#ifdef FW_BUILD_MAC
FW_PlatformError SL_API	FW_PrivPicture_MacLock(FW_HPicture picture)
{
	FW_RETURN_ERR_TRY
	{
		picture->MacLock();
	}
	FW_RETURN_ERR_CATCH
}

FW_PlatformError SL_API	FW_PrivPicture_MacUnlock(FW_HPicture picture)
{
	FW_RETURN_ERR_TRY
	{
		picture->MacUnlock();
	}
	FW_RETURN_ERR_CATCH
}

#endif

// Streaming

FW_HPicture			SL_API	FW_PrivPicture_Read(FW_HReadableStream hStream, FW_PlatformError* error)
{
	FW_ERR_TRY
	{
		FW_CReadableStream stream(hStream);
		return FW_NEW(FW_CPrivPictureRep, (stream));
	}
	FW_ERR_CATCH
	return 0;
}

void				SL_API	FW_PrivPicture_Write(
								FW_HPicture 		picture,
								FW_HWritableStream	hStream,
								FW_PlatformError* 	error)
{
	FW_ERR_TRY
	{
		FW_CWritableStream stream(hStream);
		picture->Write(stream);
	}
	FW_ERR_CATCH
}

